body {
    font-size: large;
}

button {
    padding: 3pt;
    background-color: rgb(200, 200, 200);
    border: solid;
    border-width: 1pt;
    border-bottom-left-radius: 15% 50%;
    border-bottom-right-radius: 15% 50%;
    border-top-left-radius: 15% 50%;
    border-top-right-radius: 15% 50%;
}

#progressBar {
    width: 30%;
}

#picture {
    width: 30%;
}

#gif {
    display: none;
}

#gifImage {
    width: 30%;
}

.downloadLink {
    text-align: center;
    text-decoration: none;
    color: black;
    padding: 10pt;
    background-color: rgb(200, 200, 200);
    border: solid;
    border-width: 1pt;
    border-bottom-left-radius: 15% 50%;
    border-bottom-right-radius: 15% 50%;
    border-top-left-radius: 15% 50%;
    border-top-right-radius: 15% 50%;
}